#!/bin/bash

# this script move data from current mysql to the new mysql, and update the amiserv configuration file
# please note, the user defined in /etc/cpbx/apply_patches.conf MUST be able to create users and database on the new server.
# this script will: 
# 1. read current amiserv conf
# 2. dump current database
# 3. read apply_patches.conf
# 4. restore databse on external server
# 5. Edit amiserv config
# 6. create users on external server


systemctl stop amiserv5

function showError() {
  echo "Error : $1"
  systemctl restart amiserv5
  exit 1
}

echo "Reading local configuration"
# Step 1: get database config from amiserv conf file.
dos2unix /etc/amiserv5/Config.txt
CURRENT_DBHOST=`cat /etc/amiserv5/Config.txt | grep "MYSQLIP" | cut -d '=' -f 2 | xargs`
CURRENT_DBPORT=`cat /etc/amiserv5/Config.txt | grep "MYSQLPORT" | cut -d '=' -f 2 | xargs`
CURRENT_DBUSER=`cat /etc/amiserv5/Config.txt | grep "MYSQLUSER" | cut -d '=' -f 2 | xargs`
CURRENT_DBPASSWORD=`cat /etc/amiserv5/Config.txt | grep "MYSQLPASSWORD" | cut -d '=' -f 2 | xargs`
# When using localhost, fallback to root user without password
if [ "$CURRENT_DBHOST" = "127.0.0.1" ] || [ "$CURRENT_DBHOST" = "localhost" ]
then
  echo "Localhost used, falling back to root user for DB authentication"
  CURRENT_DBUSER="root"
  CURRENT_DBPASSWORD=""
fi
if [ "$CURRENT_DBPASSWORD" != "" ]
then
  CURRENT_MYSQLDUMPCMD="mysqldump -h $CURRENT_DBHOST -P $CURRENT_DBPORT -u $CURRENT_DBUSER -p$CURRENT_DBPASSWORD"
  echo "Mysql host is $CURRENT_DBHOST:$CURRENT_DBPORT, using user $CURRENT_DBUSER with password $CURRENT_DBPASSWORD"
else
  CURRENT_MYSQLDUMPCMD="mysqldump -h $CURRENT_DBHOST -P $CURRENT_DBPORT -u $CURRENT_DBUSER"
  echo "Mysql host is $CURRENT_DBHOST:$CURRENT_DBPORT, using user $CURRENT_DBUSER"
fi

# Step 2: dump database
echo "Dumping locale database"
DUMPFILE=`mktemp`
$CURRENT_MYSQLDUMPCMD --databases amiserv5 supervision3 chatplugin > $DUMPFILE
if [ "$?" != "0" ]
then
  showError "Step 2: error during dump"
fi

# Step 3: read apply_patches configuration file
echo "Reading distant configuration"
NEW_DBHOST=`cat /etc/cpbx/apply_patches.conf | grep "hostname" | cut -d '=' -f 2 | xargs`
NEW_DBUSER=`cat /etc/cpbx/apply_patches.conf | grep "username" | cut -d '=' -f 2 | xargs`
NEW_DBPASSWORD=`cat /etc/cpbx/apply_patches.conf | grep "password" | cut -d '=' -f 2 | xargs`
if [ "$NEW_DBHOST" == "" ] || [ "$NEW_DBUSER" == "" ] || [ "$NEW_DBPASSWORD" == "" ]
then
  showError "Step 3: Invalid configuration found"
fi
echo "Distant Mysql host is $NEW_DBHOST, using user $NEW_DBUSER with password $NEW_DBPASSWORD"

# Step 4: restore the db on the new server
echo "Restoring data to distant server"
mysql -h $NEW_DBHOST -u $NEW_DBUSER -p$NEW_DBPASSWORD < $DUMPFILE
if [ "$?" != "0" ]
then
  showError "Step 4: error during restore"
fi

# Step 5 : Edit amiserv config
echo "Editing locale configuration file"
if [ -f /etc/amiserv5/Config.txt.bak ]
then
  rm -f /etc/amiserv5/Config.txt.bak
fi
cp /etc/amiserv5/Config.txt /etc/amiserv5/Config.txt.bak
sed -i "s/^MYSQLIP.*/MYSQLIP = $NEW_DBHOST/" /etc/amiserv5/Config.txt

# Step 6 : create user on external server
echo "Creating DB User"
mysql -h $NEW_DBHOST -u $NEW_DBUSER -p$NEW_DBPASSWORD <<EOF
CREATE USER "amiserv5"@"localhost" IDENTIFIED BY "SCvhgjzsSdvhvS4";
CREATE USER "amiserv5"@"127.0.0.1" IDENTIFIED BY "SCvhgjzsSdvhvS4";
CREATE USER "amiserv5"@"::1" IDENTIFIED BY "SCvhgjzsSdvhvS4";
CREATE USER "amiserv5"@"%" IDENTIFIED BY "SCvhgjzsSdvhvS4";
GRANT SELECT,DELETE,UPDATE,INSERT ON amiserv5.* TO "amiserv5"@"localhost";
GRANT SELECT,DELETE,UPDATE,INSERT ON amiserv5.* TO "amiserv5"@"127.0.0.1";
GRANT SELECT,DELETE,UPDATE,INSERT ON amiserv5.* TO "amiserv5"@"::1";
GRANT SELECT,DELETE,UPDATE,INSERT ON amiserv5.* TO "amiserv5"@"%";
GRANT SELECT,DELETE,UPDATE,INSERT ON asterisk.* TO "amiserv5"@"localhost";
GRANT SELECT,DELETE,UPDATE,INSERT ON asterisk.* TO "amiserv5"@"127.0.0.1";
GRANT SELECT,DELETE,UPDATE,INSERT ON asterisk.* TO "amiserv5"@"::1";
GRANT SELECT,DELETE,UPDATE,INSERT ON asterisk.* TO "amiserv5"@"%";
GRANT SELECT,DELETE,UPDATE,INSERT ON ombutel.* TO "amiserv5"@"localhost";
GRANT SELECT,DELETE,UPDATE,INSERT ON ombutel.* TO "amiserv5"@"127.0.0.1";
GRANT SELECT,DELETE,UPDATE,INSERT ON ombutel.* TO "amiserv5"@"::1";
GRANT SELECT,DELETE,UPDATE,INSERT ON ombutel.* TO "amiserv5"@"%"";
GRANT SELECT,DELETE,UPDATE,INSERT ON supervision3.* TO "amiserv5"@"localhost";
GRANT SELECT,DELETE,UPDATE,INSERT ON supervision3.* TO "amiserv5"@"127.0.0.1";
GRANT SELECT,DELETE,UPDATE,INSERT ON supervision3.* TO "amiserv5"@"::1";
GRANT SELECT,DELETE,UPDATE,INSERT ON supervision3.* TO "amiserv5"@"%";
GRANT SELECT,DELETE,UPDATE,INSERT ON chatplugin.* TO "amiserv5"@"localhost";
GRANT SELECT,DELETE,UPDATE,INSERT ON chatplugin.* TO "amiserv5"@"127.0.0.1";
GRANT SELECT,DELETE,UPDATE,INSERT ON chatplugin.* TO "amiserv5"@"::1";
GRANT SELECT,DELETE,UPDATE,INSERT ON chatplugin.* TO "amiserv5"@"%";
GRANT SELECT,DELETE,UPDATE,INSERT ON performance2.* TO "amiserv5"@"localhost";
GRANT SELECT,DELETE,UPDATE,INSERT ON performance2.* TO "amiserv5"@"127.0.0.1";
GRANT SELECT,DELETE,UPDATE,INSERT ON performance2.* TO "amiserv5"@"::1";
GRANT SELECT,DELETE,UPDATE,INSERT ON performance2.* TO "amiserv5"@"%";
FLUSH PRIVILEGES;
EOF

echo "Finished"
systemctl restart amiserv5
exit 0